#pragma once
#ifndef __RAYTRACE_H
#define __RAYTRACE_H
#include "Def.h"
#include "Texture.h"
#include "Blob.h"
#include <vector>

struct material {
    enum {
        gouraud=0,
        noise=1,
        marble=2,
        turbulence=3
    } type;

    color diffuse;
    //Deuxime couleur diffuse optionnelle pour les matriaux procduraux
    color diffuse2; 
    float bump, reflection, refraction, density;
    color specular;
    float power;
};

struct sphere {
	point pos;
	float size;
	int materialId;
};

struct light {
	point pos;
	color intensity;
};

struct perspective {
    enum {
        orthogonal = 0,
        conic = 1
    } type;

    float FOV;
    float clearPoint;
    float dispersion;
};

struct scene {
    std::vector<material> matTab;
	std::vector<sphere>   sphTab;
	std::vector<blob>     blbTab;
	std::vector<light>    lgtTab;
	int sizex, sizey;
    cubemap               cm;
    perspective           persp;
    int                   complexity;
};

struct context {
    float fRefractionCoef;
    color cLightScattering;
    int   level;

    static const context & getDefaultAir() 
    {
        static context airContext = {1.0f, {0.0f,0.0f,0.0f}, 0};
        return airContext;
    };
};

#define invsqrtf(x) (1.0f / sqrtf(x))

#endif // __RAYTRACE_H
