#include "Scene.h"
#include "Config.h"
#include "Raytrace.h"
#include <iostream>
using namespace std;

const vecteur NullVector = { 0.0f,0.0f,0.0f };
const point Origin = { 0.0f,0.0f,0.0f };
const SimpleString emptyString("");

void GetMaterial(const Config &sceneFile, material &currentMat)
{
    SimpleString materialType = sceneFile.GetByNameAsString("Type", emptyString);

    if (materialType.compare("turbulence") == 0)
    {
        currentMat.type = material::turbulence;
    }
    else if (materialType.compare("marble") == 0)
    {
        currentMat.type = material::marble;
    }
    else if (materialType.compare("noise") == 0)
    {
        currentMat.type = material::noise;
    }
    else
    { 
        // default
        currentMat.type = material::gouraud;
    }

    currentMat.diffuse.blue =  float(sceneFile.GetByNameAsFloat("Diffuse.Blue", 0.0f)); 
    currentMat.diffuse.red  =  float(sceneFile.GetByNameAsFloat("Diffuse.Red", 0.0f)); 
    currentMat.diffuse.green = float(sceneFile.GetByNameAsFloat("Diffuse.Green", 1.0f)); 

    currentMat.diffuse2.blue =  float(sceneFile.GetByNameAsFloat("Diffuse2.Blue", 0.0f)); 
    currentMat.diffuse2.red  =  float(sceneFile.GetByNameAsFloat("Diffuse2.Red", 0.0f)); 
    currentMat.diffuse2.green = float(sceneFile.GetByNameAsFloat("Diffuse2.Green", 1.0f)); 

    currentMat.reflection = float(sceneFile.GetByNameAsFloat("Reflection", 0.0f));
    currentMat.refraction = float(sceneFile.GetByNameAsFloat("Refraction", 0.0f));
    currentMat.density = float(sceneFile.GetByNameAsFloat("Density", 1.0f));

    currentMat.bump = float(sceneFile.GetByNameAsFloat("Bumplevel", 0.0f));


    currentMat.specular.blue = float(sceneFile.GetByNameAsFloat("Specular.Blue", 0.0f));
    currentMat.specular.red = float(sceneFile.GetByNameAsFloat("Specular.Red", 0.0f));
    currentMat.specular.green = float(sceneFile.GetByNameAsFloat("Specular.Green", 0.0f));

    currentMat.power = float(sceneFile.GetByNameAsFloat("Power", 0.0f));
}

void GetSphere(const Config &sceneFile, sphere &currentSph)
{
    currentSph.pos = sceneFile.GetByNameAsPoint("Center", Origin); 

    currentSph.size =  float(sceneFile.GetByNameAsFloat("Size", 0.0f)); 

    currentSph.materialId = sceneFile.GetByNameAsInteger("Material.Id", 0); 
}

void GetBlob(const Config &sceneFile, blob &currentBlb)
{
    currentBlb.centerList.resize(3);
    currentBlb.centerList[0] = sceneFile.GetByNameAsPoint("Center0", Origin); 
    currentBlb.centerList[1] = sceneFile.GetByNameAsPoint("Center1", Origin); 
    currentBlb.centerList[2] = sceneFile.GetByNameAsPoint("Center2", Origin); 

    currentBlb.size =  float(sceneFile.GetByNameAsFloat("Size", 0.0f)); 

    currentBlb.materialId = sceneFile.GetByNameAsInteger("Material.Id", 0); 
}

void GetLight(const Config &sceneFile, light &currentLight)
{
    currentLight.pos = sceneFile.GetByNameAsPoint("Position", Origin); 

    currentLight.intensity.blue =  float(sceneFile.GetByNameAsFloat("Intensity.Blue", 1.0f)); 
    currentLight.intensity.red  =  float(sceneFile.GetByNameAsFloat("Intensity.Red", 1.0f)); 
    currentLight.intensity.green = float(sceneFile.GetByNameAsFloat("Intensity.Green", 1.0f)); 
}

bool init(char* inputName, scene &myScene)
{
	int nbMats, nbSpheres, nbBlobs, nbLights;
	int i;
	Config sceneFile(inputName);
    if (sceneFile.SetSection("Scene") == -1)
    {
        cout << "Fichier scene mal form" << endl;
		return false;
    }
    myScene.sizex = sceneFile.GetByNameAsInteger("ImageWidth", 640);
    myScene.sizey = sceneFile.GetByNameAsInteger("ImageHeight", 480);

    myScene.complexity = sceneFile.GetByNameAsInteger("Complexity", 1);

    nbMats = sceneFile.GetByNameAsInteger("NumberOfMaterials", 0);
    nbSpheres = sceneFile.GetByNameAsInteger("NumberOfSpheres", 0);
    nbBlobs = sceneFile.GetByNameAsInteger("NumberOfBlobs", 0);
    nbLights = sceneFile.GetByNameAsInteger("NumberOfLights", 0);

    myScene.cm.name[cubemap::up] = sceneFile.GetByNameAsString("Cubemap.Up", emptyString);
    myScene.cm.name[cubemap::down] = sceneFile.GetByNameAsString("Cubemap.Down", emptyString);
    myScene.cm.name[cubemap::right] = sceneFile.GetByNameAsString("Cubemap.Right", emptyString);
    myScene.cm.name[cubemap::left] = sceneFile.GetByNameAsString("Cubemap.Left", emptyString);
    myScene.cm.name[cubemap::forward] = sceneFile.GetByNameAsString("Cubemap.Forward", emptyString);
    myScene.cm.name[cubemap::backward] = sceneFile.GetByNameAsString("Cubemap.Backward", emptyString);
    myScene.cm.bExposed = sceneFile.GetByNameAsBoolean("Cubemap.Exposed", false);
    myScene.cm.bsRGB = sceneFile.GetByNameAsBoolean("Cubemap.sRGB", false);
    myScene.cm.exposure = float(sceneFile.GetByNameAsFloat("Cubemap.Exposure", 1.0f));

    if (!myScene.cm.Init())
    {
        cout << "Pas de fichier skybox trouv" << endl;
    }

    {
        SimpleString perspectiveType = sceneFile.GetByNameAsString("Perspective.Type", emptyString);
        if (perspectiveType.compare("conic") == 0)
        {
            myScene.persp.type = perspective::conic;
            myScene.persp.FOV = float(sceneFile.GetByNameAsFloat("Perspective.FOV", 45.0f));
            myScene.persp.clearPoint = float(sceneFile.GetByNameAsFloat("Perspective.ClearPoint", 1.0f));
            myScene.persp.dispersion = float(sceneFile.GetByNameAsFloat("Perspective.Dispersion", 0.0f));
        }
        else
        { 
            // default
            myScene.persp.type = perspective::orthogonal;
        }
    }

	myScene.matTab.resize(nbMats);
	myScene.sphTab.resize(nbSpheres);
	myScene.blbTab.resize(nbBlobs);
	myScene.lgtTab.resize(nbLights);

	for (i=0; i<nbMats; ++i)
    {   
        material &currentMat = myScene.matTab[i];
        SimpleString sectionName("Material");
        sectionName.append((unsigned long) i);
        if (sceneFile.SetSection( sectionName ) == -1)
        {
            cout << "Fichier scene mal form" << endl;
		    return false;
        }
        GetMaterial(sceneFile, currentMat);
    }
	for (i=0; i<nbSpheres; ++i)
    {   
        sphere &currentSphere = myScene.sphTab[i];
        SimpleString sectionName("Sphere");
        sectionName.append((unsigned long) i);
        if (sceneFile.SetSection( sectionName ) == -1)
        {
            cout << "Fichier scene mal form" << endl;
		    return false;
        }
        GetSphere(sceneFile, currentSphere);
        if (currentSphere.materialId >= nbMats)
        {
            cout << "Fichier scene mal form" << endl;
		    return false;
        }

    }
	for (i=0; i<nbBlobs; ++i)
    {   
        blob &currentBlob = myScene.blbTab[i];
        SimpleString sectionName("Blob");
        sectionName.append((unsigned long) i);
        if (sceneFile.SetSection( sectionName ) == -1)
        {
            cout << "Fichier scene mal form" << endl;
		    return false;
        }
        GetBlob(sceneFile, currentBlob);
        if (currentBlob.materialId >= nbMats)
        {
            cout << "Fichier scene mal form" << endl;
		    return false;
        }
    }
	for (i=0; i<nbLights; ++i)
    {   
        light &currentLight = myScene.lgtTab[i];
        SimpleString sectionName("Light");
        sectionName.append((unsigned long) i);
        if (sceneFile.SetSection( sectionName ) == -1)
        {
            cout << "Fichier scene mal form" << endl;
		    return false;
        }
        GetLight(sceneFile, currentLight);
        
    }
	return true;
}

